#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <math.h>
//#define NDEBUG
#include <assert.h>

#include "vector.h"
#include "matrix.h"
#include "math.h"

// Default constructor for class MatrixInt
MatrixInt::MatrixInt(int matrixRow, int matrixCol)
{
  int i;
  row = matrixRow;
  col = matrixCol;
  ptr = new int*[row];
  assert(ptr != 0);
  for (i = 0; i < row; i++) {
    ptr[i] = new int[col];
    assert(ptr[i] != 0);
  }

  for (i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      ptr[i][j] = 0;
}

// Copy constructor for class MatrixInt
MatrixInt::MatrixInt(const MatrixInt &init)
{
  int i;
  row = init.row;
  col = init.col;
  ptr = new int*[row];
  assert(ptr != 0);
  for (i = 0; i < row; i++) {
    ptr[i] = new int[col];
    assert(ptr[i] != 0);
  }

  for (i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      ptr[i][j] = init.ptr[i][j];
}

// Destructor for class MatrixInt
MatrixInt::~MatrixInt()
{
  for (int i = 0; i < (*this).row; i++)
    delete[] ptr[i];
  delete[] ptr;
}

// Get the row number of the matrix
int MatrixInt::getRow() const { return row; }

// Get the column number of the matrix
int MatrixInt::getCol() const { return col; }

// Overloaded assignment operator
const MatrixInt &MatrixInt::operator=(const MatrixInt &right)
{
  int i;
  if (&right != this) {
    for (i = 0; i < (*this).row; i++)
      delete[] ptr[i];
    delete[] ptr;

    row = right.row;
    col = right.col;
    ptr = new int*[row];
    assert(ptr != 0);
    for (int i = 0; i < row; i++) {
      ptr[i] = new int[col];
      assert(ptr[i] != 0);
    }

    for (i = 0; i < row; i++)
      for (int j = 0; j < col; j++)
        ptr[i][j] = right.ptr[i][j];
  }

  return *this;    // enables x = y = z;
}

// Overloaded row subscript operator
int * &MatrixInt::operator[](int rowSubscript) const
{
  assert(0 <= rowSubscript && rowSubscript < row);

  return ptr[rowSubscript];
}

// Overloaded subscript operator
int &MatrixInt::operator()(int rowSubscript, int colSubscript) const
{
  assert(0 <= rowSubscript && rowSubscript < row);
  assert(0 <= colSubscript && colSubscript < col);

  return ptr[rowSubscript][colSubscript];
}

// return submatrix
MatrixInt MatrixInt::operator()(int rowLo, int rowHi, int colLo, int colHi) const
{
  assert (0 <= rowLo && rowHi < row && 0 <= colLo && colHi < col);
  MatrixInt newMatrix(rowHi - rowLo + 1, colHi - colLo + 1);

  for (int i = 0; i < rowHi - rowLo + 1; i++)
    for (int j = 0; j < colHi - colLo + 1; j++)
      newMatrix.ptr[i][j] = ptr[rowLo + i][colLo + j];

  return newMatrix;
}

// Overloaded + operator to add integer "element-wise" to MatrixInt
MatrixInt MatrixInt::operator+(int a) const
{
  MatrixInt newMatrix(row, col);

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      newMatrix.ptr[i][j] = ptr[i][j] + a;

  return newMatrix;
}

// Overloaded * operator to multiply Matrix "element-wise" by integer
MatrixInt MatrixInt::operator*(int a) const
{
  MatrixInt newMatrix(row, col);

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      newMatrix.ptr[i][j] = ptr[i][j] * a;

  return newMatrix;
}

// Overloaded output operator
ostream &operator<<(ostream &output, const MatrixInt &m)
{
  int i, j;

  for (i = 0; i < m.row; i++) {
    for (j = 0; j < m.col; j++) {
      output << m.ptr[i][j] << ' ';

      if ((j + 1) % 10 == 0)
        output << endl;
    }

    if (j % 10 != 0)
      output << endl;
  }

  return output;    // enables cout << x << y;
}

// Overloaded + operator to add integer "element-wise" to MatrixInt
MatrixInt operator+(int a, const MatrixInt &matrix)
{
  MatrixInt newMatrix(matrix);

  for (int i = 0; i < newMatrix.row; i++)
    for (int j = 0; j < newMatrix.col; j++)
      newMatrix.ptr[i][j] += a;

  return newMatrix;
}

// Overloaded * operator to multiply MatrixInt "element-wise" by integer
MatrixInt operator*(int a, const MatrixInt &matrix)
{
  MatrixInt newMatrix(matrix);

  for (int i = 0; i < newMatrix.row; i++)
    for (int j = 0; j < newMatrix.col; j++)
      newMatrix.ptr[i][j] *= a;

  return newMatrix;
}

// Overloaded + operator to add two matrices element-wise
MatrixInt operator+(const MatrixInt &matrix1, const MatrixInt &matrix2)
{
  assert(matrix1.getRow() == matrix2.getRow());
  assert(matrix1.getCol() == matrix2.getCol());

  MatrixInt newMatrix(matrix1);
  for (int i = 0; i < matrix1.getRow(); i++)
    for (int j = 0; j < matrix1.getCol(); j++)
      newMatrix.ptr[i][j] += matrix2.ptr[i][j];

  return(newMatrix);
}

// Overloaded - operator to subtract one matrix from another element-wise
MatrixInt operator-(const MatrixInt &matrix1, const MatrixInt &matrix2)
{
  assert(matrix1.getRow() == matrix2.getRow());
  assert(matrix1.getCol() == matrix2.getCol());

  MatrixInt newMatrix(matrix1);
  for (int i = 0; i < matrix1.getRow(); i++)
    for (int j = 0; j < matrix1.getCol(); j++)
      newMatrix.ptr[i][j] -= matrix2.ptr[i][j];

  return(newMatrix);
}

// Overloaded * operator to multiply two matrices element-wise
MatrixInt operator*(const MatrixInt &matrix1, const MatrixInt &matrix2)
{
  assert(matrix1.getRow() == matrix2.getRow());
  assert(matrix1.getCol() == matrix2.getCol());

  MatrixInt newMatrix(matrix1);
  for (int i = 0; i < matrix1.getRow(); i++)
    for (int j = 0; j < matrix1.getCol(); j++)
      newMatrix.ptr[i][j] *= matrix2.ptr[i][j];

  return(newMatrix);
}

// Overloaded / operator to divide one matrix by another element-wise
MatrixInt operator/(const MatrixInt &matrix1, const MatrixInt &matrix2)
{
  assert(matrix1.getRow() == matrix2.getRow());
  assert(matrix1.getCol() == matrix2.getCol());

  MatrixInt newMatrix(matrix1);
  for (int i = 0; i < matrix1.getRow(); i++)
    for (int j = 0; j < matrix1.getCol(); j++)
      newMatrix.ptr[i][j] /= matrix2.ptr[i][j];

  return(newMatrix);
}

// Overloaded % operator to multiply two matrices in usual sense
MatrixInt operator%(const MatrixInt &matrix1, const MatrixInt &matrix2)
{
  assert (matrix1.col == matrix2.row);

  MatrixInt newMatrix(matrix1.row, matrix2.col);

  for (int i = 0; i < matrix1.row; i++)
    for (int j = 0; j < matrix2.col; j++) {
      newMatrix.ptr[i][j] = 0;
      for (int k = 0; k < matrix1.col; k++)
        newMatrix.ptr[i][j] += matrix1.ptr[i][k] * matrix2.ptr[k][j];
    }

  return newMatrix;
}

// Function nullMatrixInt returns a matrix full of zeroes
MatrixInt nullMatrixInt(int row, int col)
{
  MatrixInt newMatrix(row, col);

  return(newMatrix);
}

// Function t returns the transpose of a matrix
MatrixInt t(const MatrixInt &matrix)
{
  MatrixInt newMatrix(matrix.col, matrix.row);

  for (int i = 0; i < matrix.row; i++)
    for (int j = 0; j < matrix.col; j++)
      newMatrix.ptr[j][i] = matrix.ptr[i][j];

  return newMatrix;
}

// Function cbind pastes two matrices together columnwise
MatrixInt cbind(const MatrixInt &matrix1, const MatrixInt &matrix2)
{
  assert(matrix1.row == matrix2.row);

  int i, j;
  MatrixInt newMatrix(matrix1.row, matrix1.col + matrix2.col);

  for (i = 0; i < matrix1.row; i++) {
    for (j = 0; j < matrix1.col; j++)
      newMatrix.ptr[i][j] = matrix1.ptr[i][j];
    for (j = 0; j < matrix2.col; j++)
      newMatrix.ptr[i][matrix1.col + j] = matrix2.ptr[i][j];
  }

  return newMatrix;
}

// Function rbind pastes two matrices together rowwise
MatrixInt rbind(const MatrixInt &matrix1, const MatrixInt &matrix2)
{
  assert(matrix1.col == matrix2.col);

  int i, j;
  MatrixInt newMatrix(matrix1.row + matrix2.row, matrix1.col);

  for (i = 0; i < matrix1.row; i++)
    for (j = 0; j < matrix1.col; j++)
      newMatrix.ptr[i][j] = matrix1.ptr[i][j];
  for (i = 0; i < matrix2.row; i++)
    for (j = 0; j < matrix1.col; j++)
      newMatrix.ptr[matrix1.row + i][j] = matrix2.ptr[i][j];

  return newMatrix;
}

// Function assignRow assigns a vector to a row of a matrix
void assignRow(MatrixInt &matrix, int rowNumber, const VectorInt &vec)
{
  assert(0 <= rowNumber && rowNumber < matrix.row);
  assert(matrix.col == vec.getSize());

  for (int j = 0; j < matrix.col; j++)
    matrix.ptr[rowNumber][j] = vec[j];
}

// Function assignCol assigns a vector to a column of a matrix
void assignCol(MatrixInt &matrix, int colNumber, const VectorInt &vec)
{
  assert(0 <= colNumber && colNumber < matrix.col);
  assert(matrix.row == vec.getSize());

  for (int i = 0; i < matrix.row; i++)
    matrix.ptr[i][colNumber] = vec[i];
}

// Function extractRow assigns a row of a matrix to a vector
VectorInt extractRow(const MatrixInt &matrix, int rowNumber)
{
  assert(0 <= rowNumber && rowNumber < matrix.row);
  VectorInt vec(matrix.getCol());

  for (int i = 0; i < matrix.getCol(); i++)
    vec[i] = matrix(rowNumber, i);

  return(vec);
}

// Function extractCol assigns a column of a matrix to a vector
VectorInt extractCol(const MatrixInt &matrix, int colNumber)
{
  assert(0 <= colNumber && colNumber < matrix.col);
  VectorInt vec(matrix.getRow());

  for (int i = 0; i < matrix.getRow(); i++)
    vec[i] = matrix(i, colNumber);

  return(vec);
}

// Function extractRows assigns a sequence of rows to a (smaller) matrix
MatrixInt extractRows(const MatrixInt &matrix, int rowLow, int rowHigh)
{
  assert(0 <= rowLow && rowHigh <= matrix.getRow() - 1 && rowLow <= rowHigh);
  MatrixInt newMatrix(rowHigh - rowLow + 1, matrix.getCol());

  for (int i = 0; i < rowHigh - rowLow + 1; i++)
    for (int j = 0; j < matrix.getCol(); j++)
      newMatrix(i, j) = matrix(rowLow + i, j);

  return(newMatrix);
}

// Function extractCols assigns a sequence of colums to a (smaller) matrix
MatrixInt extractCols(const MatrixInt &matrix, int colLow, int colHigh)
{
  assert(0 <= colLow && colHigh <= matrix.getCol() - 1 && colLow <= colHigh);
  MatrixInt newMatrix(matrix.getRow(), colHigh - colLow + 1);

  for (int i = 0; i < matrix.getRow(); i++)
    for (int j = 0; j < colHigh - colLow + 1; j++)
      newMatrix(i, j) = matrix(i, colLow + j);

  return(newMatrix);
}



// Default constructor for class MatrixDouble
MatrixDouble::MatrixDouble(int matrixRow, int matrixCol)
{
  int i;
  row = matrixRow;
  col = matrixCol;
  ptr = new double*[row];
  assert(ptr != 0);
  for (i = 0; i < row; i++) {
    ptr[i] = new double[col];
    assert(ptr[i] != 0);
  }

  for (i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      ptr[i][j] = 0.0;
}

// Copy constructor for class MatrixDouble
MatrixDouble::MatrixDouble(const MatrixDouble &init)
{
  int i, j;
  row = init.row;
  col = init.col;
  ptr = new double*[row];
  assert(ptr != 0);
  for (i = 0; i < row; i++) {
    ptr[i] = new double[col];
    assert(ptr[i] != 0);
  }

  for (i = 0; i < row; i++)
    for (j = 0; j < col; j++)
      ptr[i][j] = init.ptr[i][j];
}

// Copy constructor for class MatrixDouble  from MatrixInt
MatrixDouble::MatrixDouble(MatrixInt &init)
{
  int i, j;
  row = init.getRow();
  col = init.getCol();
  ptr = new double*[row];
  assert(ptr != 0);
  for (i = 0; i < row; i++) {
    ptr[i] = new double[col];
    assert(ptr[i] != 0);
  }

  for (i = 0; i < row; i++)
    for (j = 0; j < col; j++)
      ptr[i][j] = (double) init(i,j);
}

// Copy constructor from **double
MatrixDouble::MatrixDouble(double **x, int matrixRow, int matrixCol)
{
  int i;
  row = matrixRow;
  col = matrixCol;
  ptr = new double*[row];
  assert(ptr != 0);
  for (i = 0; i < row; i++) {
    ptr[i] = new double[col];
    assert(ptr[i] != 0);
  }

  for (i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      ptr[i][j] = x[i][j];
}


// Destructor for class MatrixDouble
MatrixDouble::~MatrixDouble()
{
  for (int i = 0; i < (*this).row; i++)
    delete[] ptr[i];
  delete[] ptr;
}

// Get the row number of the matrix
int MatrixDouble::getRow() const { return row; }

// Get the column number of the matrix
int MatrixDouble::getCol() const { return col; }

// Overloaded assignment operator
const MatrixDouble &MatrixDouble::operator=(const MatrixDouble &right)
{
  int i, j;
  if (&right != this) {
    for (i = 0; i < (*this).row; i++)
      delete[] ptr[i];
    delete[] ptr;

    row = right.row;
    col = right.col;
    ptr = new double*[row];
    assert(ptr != 0);
    for (i = 0; i < row; i++) {
      ptr[i] = new double[col];
      assert(ptr[i] != 0);
    }

    for (i = 0; i < row; i++)
      for (j = 0; j < col; j++)
        ptr[i][j] = right.ptr[i][j];
    }
  else
    cout << "Attempted assignment of a matrix to itself" << endl;

  return *this;    // enables x = y = z;
}

// Overloaded assignment operator from MatrixInt
const MatrixDouble &MatrixDouble::operator=(const MatrixInt &right)
{
  int i;
  for (i = 0; i < (*this).row; i++)
    delete[] ptr[i];
  delete[] ptr;

  row = right.getRow();
  col = right.getCol();
  ptr = new double*[row];
  assert(ptr != 0);
  for (i = 0; i < row; i++) {
    ptr[i] = new double[col];
    assert(ptr[i] != 0);
  }

  for (i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      ptr[i][j] = (double) right(i,j);

  return *this;    // enables x = y = z;
}

// Overloaded += operator to add another matrix
MatrixDouble &MatrixDouble::operator+=(const MatrixDouble &right)
{
  assert (row == right.getRow());
  assert (col == right.getCol());

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      ptr[i][j] += right(i, j);

  return *this;
}

// Overloaded += operator to add scalar;
MatrixDouble &MatrixDouble::operator+=(double scalar)
{

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      ptr[i][j] += scalar;

  return *this;
}

// Overloaded *= operator to multiply by scalar;
MatrixDouble &MatrixDouble::operator*=(double scalar)
{

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      ptr[i][j] *= scalar;

  return *this;
}

// Overloaded /= operator to divide by scalar;
MatrixDouble &MatrixDouble::operator/=(double scalar)
{

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      ptr[i][j] /= scalar;

  return *this;
}

// Overloaded row subscript operator
double * &MatrixDouble::operator[](int rowSubscript) const
{
  assert(0 <= rowSubscript && rowSubscript < row);

  return ptr[rowSubscript];
}

// Overloaded subscript operator
double &MatrixDouble::operator()(int rowSubscript, int colSubscript) const
{
  assert(0 <= rowSubscript && rowSubscript < row);
  assert(0 <= colSubscript && colSubscript < col);

  return ptr[rowSubscript][colSubscript];
}

// return submatrix
MatrixDouble MatrixDouble::operator()(int rowLo, int rowHi, int colLo, int colHi) const
{
  assert (0 <= rowLo && rowHi < row && 0 <= colLo && colHi < col);
  MatrixDouble newMatrix(rowHi - rowLo + 1, colHi - colLo + 1);

  for (int i = 0; i < rowHi - rowLo + 1; i++)
    for (int j = 0; j < colHi - colLo + 1; j++)
      newMatrix.ptr[i][j] = ptr[rowLo + i][colLo + j];

  return newMatrix;
}

// select rows
MatrixDouble MatrixDouble::operator()(const VectorInt &rowInd, int dummy) const
{
  assert (0 <= min(rowInd) && max(rowInd) <= row);
  MatrixDouble newMatrix(rowInd.getSize(), col);

  for (int i = 0; i < rowInd.getSize(); i++)
    for (int j = 0; j < col; j++)
      newMatrix.ptr[i][j] = ptr[rowInd[i]][j];

  return newMatrix;
}

// select cols
MatrixDouble MatrixDouble::operator()(int dummy, const VectorInt &colInd) const
{
  assert (0 <= min(colInd) && max(colInd) <= col);
  MatrixDouble newMatrix(row, colInd.getSize());

  for (int i = 0; i < col; i++)
    for (int j = 0; j < colInd.getSize(); j++)
      newMatrix.ptr[i][j] = ptr[i][colInd[j]];

  return newMatrix;
}

// select rows and cols
MatrixDouble MatrixDouble::operator()(const VectorInt &rowInd, const VectorInt &colInd) const
{
  assert (0 <= min(rowInd) && max(rowInd) <= row);
  assert (0 <= min(colInd) && max(colInd) <= col);
  MatrixDouble newMatrix(rowInd.getSize(), colInd.getSize());

  for (int i = 0; i < rowInd.getSize(); i++)
    for (int j = 0; j < colInd.getSize(); j++)
      newMatrix.ptr[i][j] = ptr[rowInd[i]][colInd[j]];

  return newMatrix;
}


// Overloaded + operator to add double "element-wise" to MatrixDouble
MatrixDouble MatrixDouble::operator+(double a) const
{
  MatrixDouble newMatrix(row, col);

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      newMatrix.ptr[i][j] = ptr[i][j] + a;

  return newMatrix;
}

// Overloaded - operator to subtract double "element-wise" from MatrixDouble
MatrixDouble MatrixDouble::operator-(double a) const
{
  MatrixDouble newMatrix(row, col);

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      newMatrix.ptr[i][j] = ptr[i][j] - a;

  return newMatrix;
}

// Overloaded * operator to multiply Matrix "element-wise" by double
MatrixDouble MatrixDouble::operator*(double a) const
{
  MatrixDouble newMatrix(row, col);

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      newMatrix.ptr[i][j] = ptr[i][j] * a;

  return newMatrix;
}

// Overloaded / operator to divide Matrix "element-wise" by double
MatrixDouble MatrixDouble::operator/(double a) const
{
  MatrixDouble newMatrix(row, col);

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      newMatrix.ptr[i][j] = ptr[i][j] / a;

  return newMatrix;
}

// Overloaded ^operator to raise Matrix "element-wise" to a power
MatrixDouble MatrixDouble::operator^(double a) const
{
  MatrixDouble newMatrix(row, col);

  for (int i = 0; i < row; i++)
    for (int j = 0; j < col; j++)
      newMatrix.ptr[i][j] = pow(ptr[i][j], a);

  return newMatrix;
}


// Overloaded output operator
ostream &operator<<(ostream &output, const MatrixDouble &m)
{
  int i, j;

  output.flags(ios::showpoint);
  output.precision(4);

  for (i = 0; i < m.row; i++) {
    for (j = 0; j < m.col; j++) {
      output.width(8);
      output << m.ptr[i][j] << ' ';

      if ((j + 1) % 10 == 0)
        output << endl;
    }

    if (j % 10 != 0)
      output << endl;
  }

  return output;    // enables cout << x << y;
}

// Overloaded + operator to add double "element-wise" to MatrixDouble
MatrixDouble operator+(double a, const MatrixDouble &matrix)
{
  MatrixDouble newMatrix(matrix);

  for (int i = 0; i < newMatrix.getRow(); i++)
    for (int j = 0; j < newMatrix.getCol(); j++)
      newMatrix(i,j) += a;

  return newMatrix;
}

// Overloaded * operator to multiply MatrixDouble "element-wise" by double
MatrixDouble operator*(double a, const MatrixDouble &matrix)
{
  MatrixDouble newMatrix(matrix);

  for (int i = 0; i < newMatrix.row; i++)
    for (int j = 0; j < newMatrix.col; j++)
      newMatrix.ptr[i][j] *= a;

  return newMatrix;
}

// Overloaded + operator to add two matrices element-wise
MatrixDouble operator+(const MatrixDouble &matrix1, const MatrixDouble &matrix2)
{
  assert(matrix1.getRow() == matrix2.getRow());
  assert(matrix1.getCol() == matrix2.getCol());

  MatrixDouble newMatrix(matrix1);
  for (int i = 0; i < matrix1.getRow(); i++)
    for (int j = 0; j < matrix1.getCol(); j++)
      newMatrix.ptr[i][j] += matrix2.ptr[i][j];

  return(newMatrix);
}

// Overloaded - operator to subtract one matrix from another element-wise
MatrixDouble operator-(const MatrixDouble &matrix1, const MatrixDouble &matrix2)
{
  assert(matrix1.getRow() == matrix2.getRow());
  assert(matrix1.getCol() == matrix2.getCol());

  MatrixDouble newMatrix(matrix1);
  for (int i = 0; i < matrix1.getRow(); i++)
    for (int j = 0; j < matrix1.getCol(); j++)
      newMatrix.ptr[i][j] -= matrix2.ptr[i][j];

  return(newMatrix);
}

// Overloaded * operator to multiply two matrices element-wise
MatrixDouble operator*(const MatrixDouble &matrix1, const MatrixDouble &matrix2)
{
  assert(matrix1.getRow() == matrix2.getRow());
  assert(matrix1.getCol() == matrix2.getCol());

  MatrixDouble newMatrix(matrix1);
  for (int i = 0; i < matrix1.getRow(); i++)
    for (int j = 0; j < matrix1.getCol(); j++)
      newMatrix.ptr[i][j] *= matrix2.ptr[i][j];

  return(newMatrix);
}

// Overloaded / operator to divide one matrix by another element-wise
MatrixDouble operator/(const MatrixDouble &matrix1, const MatrixDouble &matrix2)
{
  assert(matrix1.getRow() == matrix2.getRow());
  assert(matrix1.getCol() == matrix2.getCol());

  MatrixDouble newMatrix(matrix1);
  for (int i = 0; i < matrix1.getRow(); i++)
    for (int j = 0; j < matrix1.getCol(); j++)
      newMatrix.ptr[i][j] /= matrix2.ptr[i][j];

  return(newMatrix);
}

// Overloaded % operator to multiply two matrices in usual sense
MatrixDouble operator%(const MatrixDouble &matrix1, const MatrixDouble &matrix2)
{
  assert (matrix1.col == matrix2.row);

  int i, j, k;
  MatrixDouble newMatrix(matrix1.row, matrix2.col);

  for (i = 0; i < matrix1.row; i++)
    for (j = 0; j < matrix2.col; j++) {
      newMatrix.ptr[i][j] = 0;
      for (k = 0; k < matrix1.col; k++)
        newMatrix.ptr[i][j] += matrix1.ptr[i][k] * matrix2.ptr[k][j];
    }

  return newMatrix;
}

// Overloaded % operator to multiply matrix with vector
VectorDouble operator%(const MatrixDouble &matrix, const VectorDouble &vector)
{
  assert (matrix.col == vector.getSize());
  int i, j;

  VectorDouble product(matrix.row);

  for (i = 0; i < matrix.row; i++) {
    product[i] = 0;
    for (j = 0; j < matrix.col; j++)
    product[i] += matrix[i][j] * vector[j];
  }

  return product;
}

// Function nullMatrixDouble returns a matrix full of zeroes
MatrixDouble nullMatrixDouble(int row, int col)
{
  MatrixDouble newMatrix(row, col);

  return(newMatrix);
}

// Function I returns identity matrix
MatrixDouble I(int dim)
{
  MatrixDouble newMatrix(dim, dim);

  for (int i = 0; i < dim; i++)
    newMatrix(i, i) = 1.0;

  return(newMatrix);
}

// Function t returns the transpose of a matrix
MatrixDouble t(const MatrixDouble &matrix)
{
  MatrixDouble newMatrix(matrix.col, matrix.row);

  for (int i = 0; i < matrix.row; i++)
    for (int j = 0; j < matrix.col; j++)
      newMatrix.ptr[j][i] = matrix.ptr[i][j];

  return newMatrix;
}

// Function cbind pastes two matrices together columnwise
MatrixDouble cbind(const MatrixDouble &matrix1, const MatrixDouble &matrix2)
{
  assert(matrix1.row == matrix2.row);

  int i, j;
  MatrixDouble newMatrix(matrix1.row, matrix1.col + matrix2.col);

  for (i = 0; i < matrix1.row; i++) {
    for (j = 0; j < matrix1.col; j++)
      newMatrix.ptr[i][j] = matrix1.ptr[i][j];
    for (j = 0; j < matrix2.col; j++)
      newMatrix.ptr[i][matrix1.col + j] = matrix2.ptr[i][j];
  }

  return newMatrix;
}

// Function cbind pastes matrix and a vector together columnwise
MatrixDouble cbind(const MatrixDouble &matrix, const VectorDouble &vector)
{
  assert(matrix.row == vector.getSize());

  MatrixDouble newMatrix(matrix.row, matrix.col + 1);

  for (int i = 0; i < matrix.row; i++) {
    for (int j = 0; j < matrix.col; j++)
      newMatrix.ptr[i][j] = matrix.ptr[i][j];
    newMatrix.ptr[i][matrix.col] = vector[i];
  }

  return newMatrix;
}

// Function cbind pastes matrix and a vector together columnwise
MatrixDouble cbind(const VectorDouble &vector, const MatrixDouble &matrix)
{
  assert(matrix.row == vector.getSize());

  MatrixDouble newMatrix(matrix.row, matrix.col + 1);

  for (int i = 0; i < matrix.row; i++) {
    for (int j = 0; j < matrix.col; j++)
      newMatrix.ptr[i][j + 1] = matrix.ptr[i][j];
    newMatrix.ptr[i][0] = vector[i];
  }

  return newMatrix;
}

// Function cbind pastes two vectors together columnwise
MatrixDouble cbind(const VectorDouble &vector1, const VectorDouble &vector2)
{
  assert(vector1.getSize() == vector2.getSize());

  MatrixDouble newMatrix(vector1.getSize(), 2);

  for (int i = 0; i < vector1.getSize(); i++) {
    newMatrix(i, 0) = vector1[i];
    newMatrix(i, 1) = vector2[i];
  }

  return newMatrix;
}

// Function cbind pastes a matrix and a double together columnwise
MatrixDouble cbind(const MatrixDouble &matrix1, double scalar)
{
  MatrixDouble newMatrix(matrix1.row, matrix1.col + 1);

  for (int i = 0; i < matrix1.row; i++) {
    for (int j = 0; j < matrix1.col; j++)
      newMatrix.ptr[i][j] = matrix1.ptr[i][j];
    newMatrix.ptr[i][matrix1.col] = scalar;
  }

  return newMatrix;
}

// Function cbind pastes a double and a matrix together columnwise
MatrixDouble cbind(double scalar, const MatrixDouble &matrix2)
{
  MatrixDouble newMatrix(matrix2.row, 1 + matrix2.col);

  for (int i = 0; i < matrix2.row; i++) {
    newMatrix.ptr[i][0] = scalar;
    for (int j = 0; j < matrix2.col; j++)
      newMatrix.ptr[i][j + 1] = matrix2.ptr[i][j];
  }

  return newMatrix;
}

// Function cbind pastes a double and a vector together columnwise
MatrixDouble cbind(double scalar, const VectorDouble &vector)
{
  MatrixDouble newMatrix(vector.getSize(), 2);

  for (int i = 0; i < vector.getSize(); i++) {
    newMatrix.ptr[i][0] = scalar;
    newMatrix.ptr[i][1] = vector[i];
  }

  return newMatrix;
}

// Function rbind pastes two matrices together rowwise
MatrixDouble rbind(const MatrixDouble &matrix1, const MatrixDouble &matrix2)
{
  assert(matrix1.col == matrix2.col);

  int i, j;
  MatrixDouble newMatrix(matrix1.row + matrix2.row, matrix1.col);

  for (i = 0; i < matrix1.row; i++)
    for (j = 0; j < matrix1.col; j++)
      newMatrix.ptr[i][j] = matrix1.ptr[i][j];
  for (i = 0; i < matrix2.row; i++)
    for (j = 0; j < matrix1.col; j++)
      newMatrix.ptr[matrix1.row + i][j] = matrix2.ptr[i][j];

  return newMatrix;
}

// Function assignRow assigns a vector to a row of a matrix
void assignRow(MatrixDouble &matrix, int rowNumber, const VectorDouble &vec)
{
  assert(0 <= rowNumber && rowNumber < matrix.row);
  assert(matrix.col == vec.getSize());

  for (int j = 0; j < matrix.col; j++)
    matrix.ptr[rowNumber][j] = vec[j];
}

// Function assignCol assigns a vector to a column of a matrix
void assignCol(MatrixDouble &matrix, int colNumber, const VectorDouble &vec)
{
  assert(0 <= colNumber && colNumber < matrix.col);
  assert(matrix.row == vec.getSize());

  for (int i = 0; i < matrix.row; i++)
    matrix.ptr[i][colNumber] = vec[i];
}

// Function assignRow assigns an int vector to a row of a matrix
void assignRow(MatrixDouble &matrix, int rowNumber, const VectorInt &vec)
{
  assert(0 <= rowNumber && rowNumber < matrix.row);
  assert(matrix.col == vec.getSize());

  for (int j = 0; j < matrix.col; j++)
    matrix.ptr[rowNumber][j] = (double) vec[j];
}

// Function assignCol assigns an int vector to a column of a matrix
void assignCol(MatrixDouble &matrix, int colNumber, const VectorInt &vec)
{
  assert(0 <= colNumber && colNumber < matrix.col);
  assert(matrix.row == vec.getSize());

  for (int i = 0; i < matrix.row; i++)
    matrix.ptr[i][colNumber] = (double) vec[i];
}

// Function assignRow assigns a real to a row of a matrix
void assignRow(MatrixDouble &matrix, int rowNumber, double x)
{
  assert(0 <= rowNumber && rowNumber < matrix.row);

  for (int j = 0; j < matrix.col; j++)
    matrix.ptr[rowNumber][j] = x;
}

// Function assignCol assigns a real to a column of a matrix
void assignCol(MatrixDouble &matrix, int colNumber, double x)
{
  assert(0 <= colNumber && colNumber < matrix.col);

  for (int i = 0; i < matrix.row; i++)
    matrix.ptr[i][colNumber] = x;
}

// Function assignRows assigns a matrix to another matrix, starting at rowNumber
void assignRows(MatrixDouble &matrix, int rowNumber, const MatrixDouble &other)
{
  assert(0 <= rowNumber && rowNumber < matrix.row);
  assert(matrix.col == other.getCol());
  assert(matrix.row - rowNumber >= other.getRow());

  for (int i = 0; i < other.getRow(); i++)
    for (int j = 0; j < matrix.col; j++)
      matrix.ptr[rowNumber + i][j] = other.ptr[i][j];
}

// Function assignCols assigns a matrix to another matrix, starting at colNumber
void assignCols(MatrixDouble &matrix, int colNumber, const MatrixDouble &other)
{
  assert(0 <= colNumber && colNumber < matrix.col);
  assert(matrix.row == other.getRow());
  assert(matrix.col - colNumber >= other.getCol());

  for (int i = 0; i < matrix.row; i++)
    for (int j = 0; j < other.getCol(); j++)
      matrix.ptr[i][colNumber + j] = other.ptr[i][j];
}

// Function assignSubRow assigns a vector to specified row, starting at colNumber
void assignSubRow(MatrixDouble &matrix,  int rowNumber, int colNumber,
                  const VectorDouble &vec)
{
  assert(0 <= rowNumber && rowNumber < matrix.row);
  assert(0 <= colNumber && colNumber < matrix.col);
  assert(matrix.col - colNumber >= vec.getSize());

  for (int j = 0; j < vec.getSize(); j++)
    matrix.ptr[rowNumber][colNumber + j] = vec[j];
}

// Function assignSubCol assigns a vector to specified column, starting at rowNumber
void assignSubCol(MatrixDouble &matrix,  int rowNumber, int colNumber,
                  const VectorDouble &vec)
{
  assert(0 <= rowNumber && rowNumber < matrix.row);
  assert(0 <= colNumber && colNumber < matrix.col);
  assert(matrix.row - rowNumber >= vec.getSize());

  int i;

  for (i = 0; i < vec.getSize(); i++)
    matrix.ptr[rowNumber + i][colNumber] = vec[i];
}

// Function assignSub assigns a matrix to submatrix with (0,0) element
// given by matrix(rowNumber, colNumber)
void assignSub(MatrixDouble &matrix,  int rowNumber, int colNumber,
               const MatrixDouble &other)
{
  assert(0 <= rowNumber && rowNumber < matrix.row);
  assert(0 <= colNumber && colNumber < matrix.col);
  assert(matrix.row - rowNumber >= other.row);
  assert(matrix.col - colNumber >= other.col);

  for (int i = 0; i < other.row; i++)
    for (int j = 0; j < other.col; j++)
      matrix.ptr[rowNumber + i][colNumber + j] = other.ptr[i][j];
}

// Function extractRow assigns a row of a matrix to a vector
VectorDouble extractRow(const MatrixDouble &matrix, int rowNumber)
{
  assert(0 <= rowNumber && rowNumber <= matrix.getRow() - 1);
  VectorDouble vec(matrix.getCol());

  for (int i = 0; i < matrix.getCol(); i++)
    vec[i] = matrix(rowNumber, i);

  return(vec);
}

// Function extractCol assigns a column of a matrix to a vector
VectorDouble extractCol(const MatrixDouble &matrix, int colNumber)
{
  assert(0 <= colNumber && colNumber <= matrix.getCol() - 1);
  VectorDouble vec(matrix.getRow());

  for (int i = 0; i < matrix.getRow(); i++)
    vec[i] = matrix(i, colNumber);

  return(vec);
}

// Function extractRows assigns a sequence of rows to a (smaller) matrix
MatrixDouble extractRows(const MatrixDouble &matrix, int rowLow, int rowHigh)
{
  assert(0 <= rowLow && rowHigh <= matrix.getRow() - 1 && rowLow <= rowHigh);
  MatrixDouble newMatrix(rowHigh - rowLow + 1, matrix.getCol());

  for (int i = 0; i < rowHigh - rowLow + 1; i++)
    for (int j = 0; j < matrix.getCol(); j++)
      newMatrix(i, j) = matrix(rowLow + i, j);

  return(newMatrix);
}

// Function extractCols assigns a sequence of colums to a (smaller) matrix
MatrixDouble extractCols(const MatrixDouble &matrix, int colLow, int colHigh)
{
  assert(0 <= colLow && colHigh <= matrix.getCol() - 1 && colLow <= colHigh);
  MatrixDouble newMatrix(matrix.getRow(), colHigh - colLow + 1);

  for (int i = 0; i < matrix.getRow(); i++)
    for (int j = 0; j < colHigh - colLow + 1; j++)
      newMatrix(i, j) = matrix(i, colLow + j);

  return(newMatrix);
}

// Function extractRows assigns a specified number of rows to a (smaller) matrix
MatrixDouble extractRows(const MatrixDouble &matrix, const VectorInt &rowNumbers)
{
  int newSize = rowNumbers.getSize();
  MatrixDouble newMatrix(newSize, matrix.getCol());

  for (int i = 0; i < newSize; i++)
    for (int j = 0; j < matrix.getCol(); j++)
      newMatrix(i, j) = matrix(rowNumbers[i], j);

  return(newMatrix);
}

// Function extractCols assigns a specified number of cols to a (smaller) matrix
MatrixDouble extractCols(const MatrixDouble &matrix, const VectorInt &colNumbers)
{
  int newSize = colNumbers.getSize();
  MatrixDouble newMatrix(matrix.getRow(), newSize);

  for (int i = 0; i < matrix.getRow(); i++)
    for (int j = 0; j < newSize; j++)
      newMatrix(i, j) = matrix(i, colNumbers[j]);

  return(newMatrix);
}

// function add adds vector row- or columnwise to matrix
MatrixDouble add(const MatrixDouble &matrix, int whichWay,
                 const VectorDouble &vector)
{
  assert(1 == whichWay || 2 == whichWay);

  int row = matrix.getRow(), col = matrix.getCol(), size = vector.getSize(),
      i, j;
  MatrixDouble result(row, col);

  if (1 == whichWay) {
    assert(row == size);
    for (i = 0; i < row; i++)
      for (j = 0; j < col; j++)
        result(i, j) = matrix(i, j) + vector[i];
  }
  else {
    assert(col == size);
    for (i = 0; i < row; i++)
      for (j = 0; j < col; j++)
        result(i, j) = matrix(i, j) + vector[j];
  }

  return result;
}

// Function diag extracts diagonal of a matrix
VectorDouble diag(const MatrixDouble &matrix)
{
  assert(matrix.getRow() == matrix.getCol());

  int dim = matrix.getRow(), i;
  assert(dim == matrix.getCol());
  VectorDouble diag(dim);

  for (i = 0; i < dim; i++) {
    diag[i] = matrix(i, i);
  }

  return diag;
}

// Function apply is the Splus analogue
// whichWay = 1 corresponds to rows, whichWay = 2 to columns
VectorDouble apply(const MatrixDouble &matrix, int whichWay,
                   double (*function)(const VectorDouble &))
{
  assert(1 == whichWay || 2 == whichWay);
  int row = matrix.getRow(), col = matrix.getCol(), i;
  if (1 == whichWay) {
    VectorDouble result(row);
    for (i = 0; i < row; i++)
      result[i] = (*function)(extractRow(matrix, i));
    return result;
  }
  else {
    VectorDouble result(col);
    for (i = 0; i < col; i++)
      result[i] = (*function)(extractCol(matrix, i));
    return result;
  }
}

// Function vMv computes vec-transpose % matrix % vec
double vMv(const MatrixDouble &matrix, const VectorDouble &vector)
{
  int p = matrix.getRow();

  assert (p == matrix.getCol() && p == vector.getSize());

  return innerProduct(vector, matrix % vector);
}

// Function vMv computes vec-transpose % matrix % vec
double vMv(const VectorDouble &vector, const MatrixDouble &matrix)
{
  int p = matrix.getRow();

  assert (p == matrix.getCol() && p == vector.getSize());

  return innerProduct(vector, matrix % vector);
}

