#ifndef _matrix_h
#define _matrix_h

#include <iostream.h>

class MatrixInt {
  friend ostream &operator<<(ostream &, const MatrixInt &);
  friend MatrixInt operator+(int, const MatrixInt &); // add integer to matrix
  friend MatrixInt operator*(int, const MatrixInt &); // multiply matrix
  friend MatrixInt operator+(const MatrixInt &, const MatrixInt &);
  friend MatrixInt operator-(const MatrixInt &, const MatrixInt &);
  friend MatrixInt operator*(const MatrixInt &, const MatrixInt &);
  friend MatrixInt operator/(const MatrixInt &, const MatrixInt &);
  friend MatrixInt operator%(const MatrixInt &, const MatrixInt &);
  friend MatrixInt nullMatrixInt(int, int);
  friend MatrixInt t(const MatrixInt &); // transpose matrix
  friend MatrixInt cbind(const MatrixInt &, const MatrixInt &);
  friend MatrixInt rbind(const MatrixInt &, const MatrixInt &);
  friend void assignRow(MatrixInt &, int, const VectorInt &); 
  friend void assignCol(MatrixInt &, int, const VectorInt &);
  friend VectorInt extractRow(const MatrixInt &, int);
  friend VectorInt extractCol(const MatrixInt &, int);
  friend MatrixInt extractRows(const MatrixInt &, int, int);
  friend MatrixInt extractCols(const MatrixInt &, int, int);
public:
  MatrixInt(int = 5, int = 5);        // default constructor
  MatrixInt(const MatrixInt &);       // copy constructor
  ~MatrixInt();                       // destructor
  int getRow() const;                 // return row number
  int getCol() const;                 // return col number
  const MatrixInt &operator=(const MatrixInt &);  // assign matrix
  int * &operator[](int) const;             // subscript row operator
  int &operator()(int, int) const;          // subscript operator
  MatrixInt operator()(int, int, int, int) const; // return submatrix
  MatrixInt operator+(int) const;     // add integer to entire matrix
  MatrixInt operator*(int) const;     // multiply entire matrix by integer
private:
  int **ptr;                          // pointer to first element of matrix
  int row;                            // row number of matrix
  int col;                            // column number of matrix
};


class MatrixDouble {
  friend ostream &operator<<(ostream &, const MatrixDouble &);
  friend MatrixDouble operator+(double, const MatrixDouble &); // add double to
  friend MatrixDouble operator*(double, const MatrixDouble &); // multiply matrix
  friend MatrixDouble operator+(const MatrixDouble &, const MatrixDouble &);
  friend MatrixDouble operator-(const MatrixDouble &, const MatrixDouble &);
  friend MatrixDouble operator*(const MatrixDouble &, const MatrixDouble &);
  friend MatrixDouble operator/(const MatrixDouble &, const MatrixDouble &);
  friend MatrixDouble operator%(const MatrixDouble &, const MatrixDouble &);
  friend VectorDouble operator%(const MatrixDouble &, const VectorDouble &);
  friend MatrixDouble nullMatrixDouble(int, int);
  friend MatrixDouble I(int);
  friend MatrixDouble t(const MatrixDouble &); // transpose matrix
  friend MatrixDouble cbind(const MatrixDouble &, const MatrixDouble &);
  friend MatrixDouble cbind(const MatrixDouble &, const VectorDouble &);
  friend MatrixDouble cbind(const VectorDouble &, const MatrixDouble &);
  friend MatrixDouble cbind(const VectorDouble &, const VectorDouble &);
  friend MatrixDouble cbind(const MatrixDouble &, double);
  friend MatrixDouble cbind(double, const MatrixDouble &);
  friend MatrixDouble cbind(double, const VectorDouble &);
  friend MatrixDouble rbind(const MatrixDouble &, const MatrixDouble &);
  friend void assignRow(MatrixDouble &, int, const VectorDouble &);
  friend void assignCol(MatrixDouble &, int, const VectorDouble &);
  friend void assignRow(MatrixDouble &, int, const VectorInt &);
  friend void assignCol(MatrixDouble &, int, const VectorInt &);
  friend void assignRow(MatrixDouble &, int, double);
  friend void assignCol(MatrixDouble &, int, double);
  friend void assignRows(MatrixDouble &, int, const MatrixDouble &);
  friend void assignCols(MatrixDouble &, int, const MatrixDouble &);
  friend void assignSubRow(MatrixDouble &, int, int, const VectorDouble &);
  friend void assignSubCol(MatrixDouble &, int, int, const VectorDouble &);
  friend void assignSub(MatrixDouble &, int, int, const MatrixDouble &);
  friend VectorDouble extractRow(const MatrixDouble &, int);
  friend VectorDouble extractCol(const MatrixDouble &, int);
  friend MatrixDouble extractRows(const MatrixDouble &, int, int);
  friend MatrixDouble extractCols(const MatrixDouble &, int, int);
  friend MatrixDouble extractRows(const MatrixDouble &, const VectorInt &);
  friend MatrixDouble extractCols(const MatrixDouble &, const VectorInt &);
  friend MatrixDouble add(const MatrixDouble &, int, const VectorDouble &);
  friend VectorDouble diag(const MatrixDouble &);
  friend VectorDouble apply(const MatrixDouble &, int, double (*)(const VectorDouble &));
  friend double vMv(const MatrixDouble &, const VectorDouble &);
  friend double vMv(const VectorDouble &, const MatrixDouble &);
public:
  MatrixDouble(int = 5, int = 5);        // default constructor
  MatrixDouble(const MatrixDouble &);    // copy constructor
  MatrixDouble(MatrixInt &);             // copy constructor from int matrix
  MatrixDouble(double **, int, int);     // copy constructor from **double
  ~MatrixDouble();                       // destructor
  int getRow() const;                    // return row number
  int getCol() const;                    // return col number
  const MatrixDouble &operator=(const MatrixDouble &); // assign matrix
  const MatrixDouble &operator=(const MatrixInt &);    // assign matrix from int
  MatrixDouble &operator+=(const MatrixDouble &); // add another Matrix
  MatrixDouble &operator+=(double);      // add a scalar element-wise
  MatrixDouble &operator*=(double);      // multiply by scalar element-wise
  MatrixDouble &operator/=(double);      // divide by scalar element-wise
  double * &operator[](int) const;       // subscript row operator
  double &operator()(int, int) const;    // subscript operator
  MatrixDouble operator()(int, int, int, int) const; // return submatrix
  MatrixDouble operator()(const VectorInt &, int) const; // select rows
  MatrixDouble operator()(int, const VectorInt &) const; // select columns
  MatrixDouble operator()(const VectorInt &, const VectorInt &) const; // select rows and columns
  MatrixDouble operator+(double) const;  // add double to entire matrix
  MatrixDouble operator-(double) const;  // subtract double from entire matrix
  MatrixDouble operator*(double) const;  // multiply entire matrix by double
  MatrixDouble operator/(double) const;  // divide entire matrix by double
  MatrixDouble operator^(double) const;  // raise matrix to a power
private:
  double **ptr;                          // pointer to first element of matrix
  int row;                               // row number of matrix
  int col;                               // column number of matrix
};

#endif 