/*****

File: sub.h
--------------
Library package with subsampling routines
for self-contained use in C++ rather than export to Splus

*****/

#ifndef _sub_h
#define _sub_h

#include "vector.h"
#include "matrix.h"
#include "random.h"
#include "boot.h"

MatrixDouble subsampling_conf(double theta, const VectorDouble &x, int b,
                double alpha, int statistic_choice = 1, int sigma_choice = 1,
                double tau_pow = 0.5, int rate_hat = 0, int iid = 0, int bc = 0,
                int with = 0, int nSub = 1000);

MatrixDouble subsampling_conf_stable(const VectorDouble &x, const VectorInt &b_vec,
                double alpha, int statistic_choice = 1, int sigma_choice = 1,
                double tau_pow = 0.5, int rate_hat = 0, int iid = 0, int bc = 0,
                int with = 0, int nSub = 1000, int ma = 4, int stable_span = 2,
                double stable_thresh = 1.2);

MatrixDouble subsampling_conf_stable_sim(int n, const VectorInt &b_vec,
                double alpha, int statistic_choice = 1, int sigma_choice = 2,
                double tau_pow = 0.5, int rate_hat = 0, int iid = 1, int bc = 0,
                int with = 0, int nSub = 1000, int numSim = 500, int ma = 2,
                int stable_span = 2, double stable_thresh = 1.2, int print = 1,
                int read_file = 1);

MatrixDouble subsampling_ar_PV_dimi_sim(int n, VectorInt b_vec, double mu,
          double beta, VectorDouble ar, double ma, double alpha, int p_fit,
          int stud = 0, int trend = 0, int num_sim = 1000,
          VectorDouble (*randgen)(int n) = rnorm, int n_start = 100);

MatrixDouble subsampling_ar_conf(const VectorDouble &x, int p, int b, double alpha,
                                 double tau_pow, int stud, int trend);

MatrixDouble subsampling_ar_conf_stable(const VectorDouble &x, int p,
                         const VectorInt b_vec, double alpha,
                         double tau_pow, int stud, int trend, int ma,
                         int stable_span, double stable_thresh);

MatrixDouble subsampling_ar_conf_stable_sim(int n, VectorInt b_vec, double mu,
          double beta, VectorDouble ar, double alpha, int stud, int trend,
          int num_sim = 1000, VectorDouble (*randgen)(int n) = rnorm,
          int ma_smoo = 3, int stable_span = 2, double stable_thresh = 1.2,
          int n_start = 100);

MatrixDouble subsampling_ar_conf_b_cal_boot(const VectorDouble &x, int p,
          const VectorInt b_vec, double alpha, double tau_pow, int stud,
          int trend, int B, int b_boot,
          VectorDouble (*boot) (const VectorDouble &, int, int, int) =
          bootstrapStat, int nStart = 100);

MatrixDouble subsampling_ar_conf_b_cal_boot_sim(int n, VectorInt b_vec,
          double mu, double beta, VectorDouble ar, double alpha, int stud = 1,
          int trend = 1, int num_sim = 1000, int B = 200, int b_boot = 5,
          VectorDouble (*randgen)(int n) = rnorm,
          VectorDouble (*boot) (const VectorDouble &, int, int, int) =
          bootstrapStat, int n_start = 100);

MatrixDouble subsampling_ar_fix_b_sim(int n, VectorInt b_vec, double mu,
          double beta, VectorDouble ar, double alpha, int stud = 1, int trend = 1,
          int num_sim = 1000, VectorDouble (*randgen)(int n) = rnorm,
          int n_start = 100);

void stock_conf_sim(int n, double mu, double beta, double *coverage_demean,
                    double *coverage_detrend, int num_sim, int n_start = 100);

#endif /* sub.h */
