/*****

File: thresh.h
--------------
Library package with threshold routines

*****/

#ifndef _thresh_h
#define _thresh_h

#include "vector.h"
#include "matrix.h"

MatrixDouble TARc(const VectorDouble &x, const VectorDouble &gammaBound, int SD,
                  int gridBound = 100);

VectorDouble TARcSub(const VectorDouble &x, int b, const VectorDouble &gammaBound,
                     const VectorDouble &theta0, int TARfit,
                     const VectorDouble &TARprevious, int gridBound);

MatrixDouble TARcSim(int n, const VectorInt &bVec, int numSim, int gridBound = 100,
                     const VectorDouble &gammaBound = c(-100.0, 40.0));

double TARate(const VectorDouble &x, int I, int J, double quantLow,
              double quantUp, double nPow, int TARfit = 1, double gammaHatPrevious = 0,
              const VectorDouble &gammaBound = c(-40.0, 40.0), int gridBound = 100);

MatrixDouble TARdc(const VectorDouble &x, const VectorDouble &gammaBound, int SD = 0,
                   int gridBound = 100);

MatrixDouble TARdcFast(const VectorDouble &x, const VectorDouble &gammaBound,
                       int SD = 0, int gridBound = 100, int H0 = 0, double gamma0 = 0);

MatrixDouble TARdcFastS(const VectorDouble &xPredictor, const VectorDouble &xResponse,
                        const VectorDouble &gammaBound, int SD = 0,
                        int gridBound = 100, int H0 = 0, double gamma0 = 0);

VectorDouble TARdcSub(const VectorDouble &x, int b, const VectorDouble &gammaBound,
                      const VectorDouble &theta0, int TARfit,
                      const VectorDouble &TARprevious, int gridBound = 100);

VectorDouble TARdcSubFast(const VectorDouble &x, int b, double tauPow,
                          const VectorDouble &gammaBound,
                          const VectorDouble &theta0, int H0, int TARfit,
                          const VectorDouble &TARprevious = vec(0), int gridBound = 100);

double TARdcSubFastcTest(const VectorDouble &x, int b,
                         const VectorDouble &gammaBound, int TARfit,
                         const VectorDouble &TARprevious = vec(0), int gridBound = 100);

VectorInt TARdcSubBsize(const VectorDouble &x, const VectorInt &bVec, double tauPow,
                        const VectorDouble &alpha, int numSim, int sameResid,
                        const VectorDouble &gammaBound = c(-40.0, 40.0),
                        int H0 = 0, double gamma0 = 0, int gridBound = 100, int thetaInd = 0);

VectorInt TARdcSubcTestBsize(const VectorDouble &x, const VectorInt &bVec,
                        const VectorDouble &alpha, int numSim, int sameResid,
                        const VectorDouble &gammaBound = c(-40.0, 40.0),
                        int gridBound = 100);

VectorDouble TARdcSim(int n, const VectorInt &bVec, int numSim, int H0,
                      int dc = 1, int tauHat = 0, int thetaInd = 0, int gridBound = 100,
                      const VectorDouble &gammaBound = c(-40.0, 40.0));

VectorDouble TARdcBoptSim(int n, const VectorInt &bVec, const VectorDouble &alpha,
                          int numSim, int numSimBsize, int sameResid,
                          int H0, int dc, int tauHat, int thetaInd = 0,
                          int gridBound = 100,
                          const VectorDouble &gammaBound = c(-40.0, 40.00));

MatrixDouble TARcTestSim(int n, const VectorInt &bVec, const VectorDouble &alpha,
                         int dc, int numSim, int outputType = 0, int numSimBsize = 200,
                         int sameResid = 1, int gridBound = 100,
                         const VectorDouble &gammaBound = c(-40.0, 40.0));

MatrixDouble TARateSim(int n, const VectorDouble &pow, int I, int J,
                       double quantLow, double quantUp, int numSim);

VectorDouble threshLS1(const MatrixDouble &x, const VectorDouble &y, const VectorDouble &q,
                      const VectorDouble &gammaBound, double gamma0, int grid = 0,
                      int N = 100);

VectorDouble threshLS(const MatrixDouble &x, const VectorDouble &y, const VectorDouble &q,
                      const VectorDouble &gammaBound, double gamma0, int grid = 0,
                      int N = 100);

MatrixDouble threshSub(const MatrixDouble &x, const VectorDouble &y, const VectorDouble &q,
                       const VectorDouble &gammaBound, const VectorDouble &alpha, int b,
                       int gamma0, int deltaInd, double delta0, int iid,
                       int iidSim, int LSfit = 1, const VectorDouble &LSprevious = vec(0),
                       int grid = 0, int N = 0);

#endif /* thresh.h */
