/*****

File: time_sim.h
--------------
Library package for generation of time series
for self-contained use in C++ rather export to Splus

*****/

#ifndef _time_sim_h
#define _time_sim_h

#include "vector.h"
#include "matrix.h"
#include "boot.h"
#include "random.h"

VectorDouble expSim(int n, int nStart = 100);

VectorDouble Markov2Sim(int n, VectorDouble piVec, int mean0 = 1, int nStart = 100);

VectorDouble rgarch11(int n, double omega, double alpha, double beta,
                      int n_start = 100);

VectorDouble armaSim(int n, double mu, const VectorDouble &ar,
                     const VectorDouble &ma, VectorDouble (*randgen)(int m) = rnorm,
                     int nStart = 100);

VectorDouble armaSim(int n, double mu, const VectorDouble &ar,
                     const VectorDouble &ma, const VectorDouble &innovations,
                     int nStart = 100);

void ar_trend_sim(int n, double mu, double beta, double *ar,
                  int n_ar, double *innov, int n_start = 100);

VectorDouble arTrendSim(int n, double mu, double beta, const VectorDouble &ar,
                        VectorDouble (*randgen)(int m) = rnorm, int n_start = 100);

VectorDouble arTrendBootSim(int n, double mu, double beta, const VectorDouble &ar,
                        VectorDouble fitted_innov, int b,
                        VectorDouble (*boot) (const VectorDouble &, int, int, int)
                        = bootstrapStat, int n_start = 100);

VectorDouble ArmaTrendSim(int n, double mu, double beta, const VectorDouble &ar,
                          double ma = 0, VectorDouble (*randgen)(int m) = rnorm,
                          int n_start = 100);

MatrixDouble VAR1SimNorm(int n, const MatrixDouble &Var, const VectorDouble &start);

MatrixDouble VAR1SimBoot(int n, const MatrixDouble &Var, const VectorDouble &start,
                         const MatrixDouble &res, int b = 1, int statBoo = 1);

MatrixDouble VAR1_LM(int n, double rho, double theta, double lambda, int n_start = 100);

VectorDouble GarchSimBoot(int n, double omega, double alpha, double beta,
                          const VectorDouble &res, int nStart = 100);

MatrixDouble DGP_LM(int n, double beta, double rho, double theta, double lambda,
                    int n_start = 100);

MatrixDouble DGP_H99(int n, double d2, int zISq, VectorDouble (*randgen)(int m) = rnorm);

VectorDouble DGP_CT98(int n, double p0 = 1, double pa = 0.6, double pb = -0.6,
                      double gamma = 0.8, double s2Mult= 2, int nStart = 100);

VectorDouble DGP_TARdc(int n, const VectorDouble &theta, double s2Mult = 1,
                       int nStart = 100);

VectorDouble DGP_TARdcBoot(int n, const VectorDouble &theta,
                           const VectorDouble &resid1, const VectorDouble &resid2,
                           int sameResid = 1, double x0 = 0, int nStart = 50);

MatrixDouble DGP_BR_Horo(int n, int uDist = 1,
                         const VectorDouble &beta = c(1/sqrt(2), 1/sqrt(2)),
                         double uMult = 1/sqrt(2));

MatrixDouble AR_Homo(int n, int p, double rho, VectorDouble (*randgen)(int m) = rnorm,
                     int nStart = 100);

MatrixDouble AR_Het1(int n, int p, double rho, VectorDouble (*randgen)(int m) = rnorm,
                     int nStart = 100);

MatrixDouble MA_Homo(int n, int p, double theta, VectorDouble (*randgen)(int m) = rnorm,
                     int nStart = 100);

#endif /* time_sim.h */
