#ifndef _vector_h
#define _vector_h

#include <iostream.h>

class VectorInt {
  friend ostream &operator<<(ostream &, const VectorInt &);
  friend VectorInt operator+(const int, const VectorInt &);
  friend VectorInt operator*(const int, const VectorInt &);
  friend VectorInt c(const VectorInt &, const VectorInt &);
  friend VectorInt c(const VectorInt &, int);
  friend VectorInt c(int, const VectorInt &);
  friend VectorInt c(int, int);
  friend VectorInt permute(const VectorInt &, const VectorInt &);
  friend VectorInt vec(int);
  friend VectorInt rep(int, int);
  friend VectorInt nullVectorInt(int);
public:
  VectorInt(int = 10);                  // default constructor
  VectorInt(const VectorInt &);         // copy constructor
  VectorInt(const int *, const int);    // copy constructor from int *
  VectorInt(const int *, const int, const int);  // copy constructor from int *
  ~VectorInt();                         // destructor
  int getSize() const;                  // return size
  const VectorInt &operator=(const VectorInt &);   // assign vectors
  VectorInt &operator+=(const VectorInt &); // add another Vector
  VectorInt &operator+=(int);           // add a scalar element-wise
  VectorInt &operator*=(int);           // multiply by scalar element-wise
  int &operator[](int) const;           // subscript operator
  VectorInt operator()(int, int) const; // return a subvector
  VectorInt operator+(int) const;       // add integer to entire vector
  VectorInt operator*(int) const;       // multiply entire vector by integer
private:
  int *ptr;                             // pointer to first element of vector
  int size;                             // size of the vector
};


class VectorDouble {
  friend ostream &operator<<(ostream &, const VectorDouble &);
  friend VectorDouble operator+(const double, const VectorDouble &);
  friend VectorDouble operator*(const double, const VectorDouble &);
  friend VectorDouble operator+(const VectorDouble &, const VectorDouble &);
  friend VectorDouble operator-(const VectorDouble &, const VectorDouble &);
  friend VectorDouble operator*(const VectorDouble &, const VectorDouble &);
  friend VectorDouble operator/(const VectorDouble &, const VectorDouble &);
  friend double innerProduct(const VectorDouble &, const VectorDouble &);
  friend double innerProduct(const VectorDouble &);
  //friend MatrixDouble outerProduct(const VectorDouble &vec1, const VectorDouble &vec2);
  friend VectorDouble c(const VectorDouble &, const VectorDouble &);
  friend VectorDouble c(const VectorDouble &, double);
  friend VectorDouble c(double, const VectorDouble &);
  friend VectorDouble c(double, double);
  friend VectorDouble permute(const VectorDouble &, const VectorInt &);
  friend VectorDouble rep(double, int);
  friend VectorDouble nullVectorDouble(int);
  friend VectorDouble vec(double);
  friend void assignSub(VectorDouble &, int, const VectorDouble &);
public:
  VectorDouble(int = 10);                  // default constructor
  VectorDouble(const VectorDouble &);      // copy constructor
  VectorDouble(const VectorInt &);         // copy constructor
  VectorDouble(const double *, const int); // copy constructor from *double
  VectorDouble(const double *, const int, const int); // copy constructor from *double
  VectorDouble(const int *, const int);    // copy constructor from *int
  VectorDouble(const int *, const int, const int); // copy constructor from *int
  ~VectorDouble();                         // destructor
  int getSize() const;                     // return size
  const VectorDouble &operator=(const VectorDouble &);   // assign vectors
  const VectorDouble &operator=(const VectorInt &);   // assign vectors
  VectorDouble &operator+=(const VectorDouble &); // add another Vector
  VectorDouble &operator+=(double);      // add a scalar element-wise
  VectorDouble &operator-=(double);      // subtract a scalar element-wise
  VectorDouble &operator*=(double);      // multiply by scalar element-wise
  VectorDouble &operator/=(double);      // divide by scalar element-wise
  double &operator[](int) const;           // subscript operator
  VectorDouble operator()(int, int) const; // return a subvector
  VectorDouble operator+(double) const;    // add double to entire vector
  VectorDouble operator-(double) const;    // subtract double from entire vector
  VectorDouble operator*(double) const;    // multiply entire vector by double
  VectorDouble operator/(double) const;    // divide entire vector by double
  VectorDouble operator^(double) const;    // raise vector to a power
  double sumSquares() const;               // computes sum of squares
private:
  double *ptr;                         // pointer to first element of vector
  int size;                            // size of the vector
};

#endif 